package	jp.co.sds_corp.util.daemon.core;

import java.io.Serializable;

import jp.co.sds_corp.util.config.IConfigSavable;
import jp.co.sds_corp.util.config.IConfigurable;
import jp.co.sds_corp.util.daemon.IService;

/**
* デーモンクラスの定義です。
*/
public interface IDaemon extends IConfigurable, IConfigSavable, ITerminatable,
					Serializable
{
	public enum Status {
		/** 状態：死亡 */
		DEAD,
		/** 状態：生存 */
		ALIVE,
		/** 状態：処理実行中 */
		RUN,
		/** 状態：異常終了 */
		ERROR,
		/** 状態：不明 */
		UNKNOWN;
	}

	/**
	* モジュールの開始
	*/
	void start();

	/**
	* モジュールの名称を設定します。
	* @param	name	モジュール名称
	*/
	void setName(String name);

	/**
	* モジュールの名称を取得します。
	* @return	モジュール名称
	*/
	String getName();

	/**
	* モジュールの状態を設定します。
	* @param	var		モジュールの状態
	*/
	void setStatus( Status var );
	/**
	* モジュールの状態をします。
	* @return	モジュールの状態
	*/
	Status getStatus();
	/**
	* モジュールの状態を文字列で取得します。
	* @return	モジュールの状態
	*/
	String getStatusString();

	/**
	* モジュールの生存問合わせ
	* @return	true:生存中 false:死亡
	*/
	boolean isAlive();

	/**
	* このデーモンが実行可能であるかどうかを返します。
	* @return    デーモンが実行可能な場合はtrue、そうでない場合はfalse
	*/
	public boolean isValid();

	/**
	* サービスを取得する
	* @return	サービス
	*/
	IService getService();
	/**
	* サービスを設定する
	* @param	var		サービス
	*/
	void setService( IService var );

}
