package	jp.co.sds_corp.util.daemon.remote;

import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.daemon.manager.DaemonManager;
import jp.co.sds_corp.util.daemon.remote.jms.JMSClient;
import jp.co.sds_corp.util.daemon.remote.jms.JMSServer;
import jp.co.sds_corp.util.daemon.remote.rmi.RMIClient;
import jp.co.sds_corp.util.daemon.remote.rmi.RMIServer;
import jp.co.sds_corp.util.daemon.remote.socket.SocketClient;
import jp.co.sds_corp.util.daemon.remote.socket.SocketServer;

/**
* リモート制御するサーバオブジェクトを生成します。
daemon.xmlで、Remote要素のtype属性またはclass属性を見て、
{@link IRemoteServer},{@link IRemoteClient}を生成します。<br>
typeは、RMI,Socket,JMSが指定可能です。
<pre>
&lt;Remote type="RMI"&gt;
	&lt;host&gt;localhost&lt;/host&gt;
	&lt;port&gt;2011&lt;/port&gt;
	&lt;name&gt;DaemonManager&lt;/name&gt;
&lt;/Remote&gt;
</pre>
*/
public class RemoteFactory
{
	public static IRemoteServer createServer( DaemonManager manager,
			IConfiguration config )
		throws Exception
	{
		IRemoteServer ret = null;
		String type = config.getAttributes().get( "type", null );
		String cls = config.getAttributes().get( "class", null );
		if ( type != null ) {
			if ( type.equals( "RMI" )) {
				ret = new RMIServer();
			} else
			if ( type.equals( "Socket" )) {
				ret = new SocketServer();
			} else
			if ( type.equals( "JMS" )) {
				ret = new JMSServer();
			} else {
				throw( new Exception( "unknown type [" + type + "]" ));
			}
		} else if ( cls != null ) {
			ret = (IRemoteServer )Class.forName( cls ).newInstance();
		}
		if ( ret == null ) {
			throw( new IllegalArgumentException( "type/class not defined." ));
		}

		ret.setManager( manager );
		ret.init( config );
		return ret;
	}

	public static IRemoteClient createClient( IConfiguration config )
		throws Exception
	{
		IRemoteClient ret = null;
		String type = config.getAttributes().get( "type", null );
		String cls = config.getAttributes().get( "class", null );
		if ( type != null ) {
			if ( type.equals( "RMI" )) {
				ret = new RMIClient();
			} else
			if ( type.equals( "Socket" )) {
				ret = new SocketClient();
			} else
			if ( type.equals( "JMS" )) {
				ret = new JMSClient();
			} else {
				throw( new Exception( "unknown type [" + type + "]" ));
			}
		} else if ( cls != null ) {
			ret = (IRemoteClient )Class.forName( cls ).newInstance();
		}
		if ( ret == null ) {
			throw( new IllegalArgumentException( "type/class not defined." ));
		}

		ret.init( config );
		return ret;
	}

}
