package	jp.co.sds_corp.util.daemon.util.msg;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.Arrays;
import java.util.Iterator;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Vector;

import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.daemon.AbstractService;
import jp.co.sds_corp.util.file.WildcardFilenameFilter;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;
import jp.co.sds_corp.util.msg.IMessage;
import jp.co.sds_corp.util.msg.IMessageAdapter;
import jp.co.sds_corp.util.msg.MessageAdapterFactory;

/**
<PRE>
<Adapter class="">
</Adapter>
<Repeat interval="300" times="1" perLine="true" />

<File dir="." names="*.txt" />
</PRE>
*/
public class MessageSender extends AbstractService
{
	static final CmnLogger logger = CmnLogFactory.getLogger();
	IMessageAdapter	adapter;
	IMessage	message;
	Vector<File>	listFile = new Vector<File>();
	long	interval = 0;
	int		times = 0;
	boolean	perLine = true;
	Timer	timer = new Timer();

	/**
	* 設定情報で初期化します。
	* @param	config	設定情報
	*/
	public void init( IConfiguration config ) throws Exception {
		IConfiguration msgCfg = config.getChild( "Adpater" );
		MessageAdapterFactory fac = new MessageAdapterFactory();
		adapter = fac.connect( msgCfg );
		adapter.createSender();
		message = adapter.createMessage( IMessage.Type.TEXT );

		IConfiguration repCfg = config.getChild( "Repeat" );
		String var;
		var = repCfg.getAttributes().get( "interval", "0" );
		interval = Long.valueOf( var ).longValue();
		var = repCfg.getAttributes().get( "times", "0" );
		times = Integer.valueOf( var ).intValue();
		var = repCfg.getAttributes().get( "perLine", "true" );
		perLine = Boolean.valueOf( var ).booleanValue();

		Iterator<IConfiguration> itr = config.getChilds( "File" );
		while ( itr.hasNext()) {
			IConfiguration c = itr.next();
			String sdir = c.getAttributes().get( "dir" );
			String name = c.getAttributes().get( "name" );

			File dir = new File( sdir );
			WildcardFilenameFilter filter = new WildcardFilenameFilter( name );
			File[] list = dir.listFiles( filter );
			Arrays.sort( list );
			for ( int i = 0; i < list.length; i++ ) {
				listFile.add( list[i] );
			}
		}
	}

	/**
	* 処理を開始します。
	*/
	public void start() {
		try {
			adapter.start();
		} catch ( Exception ex ) {
			logger.error( this, "message start error", ex );
			return;
		}
		logger.debug( this, "times=" + times + " interval=" + interval
							+ " perLine=" + perLine );
		setInterrupt( false );
		SendTask task = new SendTask();
		timer.scheduleAtFixedRate( task, 0, interval );

		for ( ;; ) {
			if ( isInterrupted()) {
				break;
			}
			try {
				Thread.sleep( 1000 );
			} catch ( InterruptedException iex ) {
				break;
			}
		}
	}

	private class SendTask extends TimerTask
	{
		int	filePos;
		int	count;
		BufferedReader	br;
		File	file;

		SendTask() {
		}

		public void run() {
			try {
				if ( br == null ) {
					file = listFile.get( filePos );
					logger.debug( this, "send file[" + file + "]" );
					br = new BufferedReader( new FileReader( file ));
				}
				send();
			} catch ( Exception ex ) {
				logger.error( this, "file open error", ex );
				countUp();
			}
		}

		void countUp() {
			try {
				br.close();
			} catch ( Exception ex ) {
			}
			br = null;
			filePos++;
			if ( filePos >= listFile.size() ) {
				filePos = 0;
				count++;
				if ( count >= times ) {
					timer.cancel();
					setInterrupt( true );
				}
			}
		}

		void send() throws Exception {
			StringBuffer buf = new StringBuffer();
			for ( ;; ) {
				String line = br.readLine();
				if ( line == null ) {
					countUp();
					if ( perLine ) {
						return;
					}
					break;
				}
				buf.append( line );
				if ( perLine ) {
					break;
				}
			}
			String msg = buf.toString();
			logger.debug( this, "[" + msg + "]" );
			message.setObject ( msg );
			adapter.writeMessage( message, 60 );
		}
	}

	/**
	* 処理を終了します。
	*/
	public void stop() {
		setInterrupt( true );
		timer.cancel();
		try {
			adapter.stop();
			adapter.close();
		} catch ( Exception ex ) {
			logger.error( this, "message stop error", ex );
		}
	}

}
